#include <avr/io.h>
#include <util/delay.h>     /* bei alter avr-libc: #include <avr/delay.h> */ 
#include "lcd.h"

#define LCD_RS PORTD.7
#define LCD_EN PORTD.6
#define LCD_DB4 PORTC.4
#define LCD_DB5 PORTC.5
#define LCD_DB6 PORTC.6
#define LCD_DB7 PORTC.7


#define LCD2LINEMODE = 1
#define LCDON = 1

#define LCDBLINK = 1
#define LCDCURSOR = 1

#define LCDSHIFT = 1
#define LCDINCREMENT = 1

void LcdInit()
{
	//Wait for Display selftest. You can remove this, if this function is called more than 30ms after start
	_delay_ms(40);
	
	//1. Function set
	//RS = ||--| = DATA
	//Send 00100 x2
	LcdSend(0b00000100);
	LcdSend(0b00000100);
	//Send 000 + DISPLAY ON/OFF + 1/2 LINE MODE
	LcdSend((LCDON << 3) | (LCD2LINEMODE << 4));
	//Wait for more than 39s
	_delay_us(40);	

	//2. Display ON/OFF Control
	//Send 0
	LcdSend(0);
	//Send 0 + LCDBLINK + LCDCURSOR + LCDON + 1
	LcdSend((LCDBLINK << 1) | (LCDCURSOR << 2) | (LCDON << 3) || (1 << 4));
	//Wait for more than 39s
	_delay_us(40);	

	//3. Display Clear
	LcdClear();
	//Wait for more than 1.53ms
	_delay_ms(2);	

	//4. Entry Mode Set
	//Send 0
	LcdSend(0);
	//Send 0 + LCDSHIFT + LCDINCREMENT
	LcdSend((LCDSHIFT << 1) | (LCDINCREMENT << 2) | (1 << 3));
	//Wait for more than 39s
	_delay_us(40);	

	//Initialization end
}

void LcdClear()
{
	LcdSend(0);
	LcdSend(0b00001000);
}

void LcdSend(uint8_t data)
{
	LCD_RS  = data & 0x01;
	LCD_DB4 = data & 0x02;
	LCD_DB5 = data & 0x04;
	LCD_DB6 = data & 0x08;
	LCD_DB7 = data & 0x10;
	LCD_EN  = 1;
	LCD_EN  = 0;
}

void LcdWrite(const char[80] txt)
{
	for(uint8_t i = 0; txt[i]; i++)
		WriteChar((uint8_t)txt[i]);
		
}
void LcdWriteChar(uint8_t c)
{
	LcdSend((c << 1) | (1 << 0));
	LcdSend((c >> 3) | (1 << 0));
}
void LcdSetCursor(int line, int x)
{

}

